# AUTOGENERATED! DO NOT EDIT! File to edit: 01_Selenium.ipynb (unless otherwise specified).

__all__ = ['save_authentification_session', 'set_auth_session_with_selenium', 'add_auth_session_web_driver',
           'load_json']

# Cell
import json

def save_authentification_session(auth_file='auth.json'):
    global driver
    authentification_session={
        'cookies': driver.get_cookies(),
        'start_url': driver.current_url,
        'user_agent': driver.execute_script('return navigator.userAgent')
    }
    json.dump(authentification_session,open(auth_file,"w"))
    return(authentification_session)

# Cell
import time
import os

def set_auth_session_with_selenium(log_in_url,auth_file='auth.json'):
    global driver

    # Import Chrome
    import undetected_chromedriver as uc
    uc.TARGET_VERSION = 86

    def driver_():
        global driver
        uc.install()
        from selenium import webdriver

        # Instanciate the web driver
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument('--disable-extensions')
        chrome_options.add_argument('--profile-directory=Default')
        chrome_options.add_argument("--disable-plugins-discovery")
        driver = webdriver.Chrome(chrome_options=chrome_options)

    try :
        driver_()
    except :
        os.remove('./chromedriver')
        driver_()


    print('You have 2 minutes to login & define start_url you want !')
    driver.get(log_in_url)

    time.sleep(2*60)

    auth=save_authentification_session(auth_file)
    driver.quit()

    return auth

# Cell

load_json= lambda file='auth.json' : json.load(open(file,'r'))

def add_auth_session_web_driver(auth_file='auth.json'):
    global driver

    auth=load_json(auth_file)

    # Request a first time the start url to append the right cookies
    driver.get(auth['start_url'])
    # Set the cookies to the right values
    _=[driver.add_cookie(c) for c in auth['cookies']]

    driver.get(auth['start_url'])


# AUTOGENERATED! DO NOT EDIT! File to edit: 05_cli.ipynb (unless otherwise specified).

__all__ = ['update_settings_requirements']

# Cell
import os
import re


def update_settings_requirements():
    if not os.path.exists('./requirements.txt'):
        raise Exception('requirements.txt does not exist, you could use pigar to augenerate one')
    if not os.path.exists('./settings.ini'):
        raise Exception('settings.ini does not exist, no settings.ini to update')
    with open('./requirements.txt','r') as f_requirements :
        requirements=f_requirements.read()

    requirements_line='requirements = '+' '.join(re.findall(r'(.*)==.*',requirements))

    with open('./settings.ini','r') as f_settings:
        settings=f_settings.read()
    with open('./settings.ini','w') as f_settings:
        f_settings.write(re.sub('(.*\n).*requirements = (.*)(\n.*)',fr'\1{requirements_line}\3',settings))


# AUTOGENERATED! DO NOT EDIT! File to edit: 03_Structure.ipynb (unless otherwise specified).

__all__ = ['MetaID', 'MongoObj']

# Cell
from bson.objectid import ObjectId
import warnings
import pandas as pd

# for storing the data localy
import kutility.core
import json
import os

from math import isnan

class MetaID(type):
    """
        Reference MongoObj with the same unique ID of MongoDB : ObjectId
    """

    _register={} # { ObjectId('str(ObjectID)'): obj }

    # ------- Data ------

    @property
    def data(cls):
        """
            Pandas Dataframe representation of the data
        """
        l_data=[obj.__Mongo_dict__ for obj in cls]
        if l_data :
            df=pd.DataFrame(l_data)
            df=df.set_index('_id',drop=False)
            df['generation_time_utc']=df._id.apply(lambda _id : ObjectId(_id).generation_time)
            return(df)

    #       Basic data queries for joining data beween classes
    def data_group_by(cls,key):
        df=cls.data
        key_ids=df[key].unique()
        return({key_id:df[df[key]==key_id] for key_id in key_ids})

    def last_data_group_by(cls,key):
        """
            last data available unique for the key
        """
        temp={key_id : df.tail(1) for key_id,df in cls.data_group_by(key).items()}
        df=pd.concat(temp.values()).set_index(key,drop=False)
        return(df)


    def add_data(cls,data,uniq_keys=['url']):
        """
            Add data from a pandas table without duplicates on uniq_keys
        """

        data_registered=cls.data

        # Approach chose : concatenation of pandas data ; suppress the right duplicates ; create the new offers

        if data_registered is None : # keep the most recent offer if no data is registered
            temp_data=pd.concat([data]).drop_duplicates(subset=uniq_keys, keep='first')
            temp_data['_id']=None
        else : # keep the first offer registered (thus the 'last' in term of time) if there is duplicates
            temp_data=pd.concat([data,data_registered]).drop_duplicates(subset=uniq_keys,keep='last')

        # Creating the new instances
        new_data=temp_data['_id'].isna() # mask on new data (not having an _id)

        l_new_data=[]
        for id_index,row in temp_data[new_data].iterrows() :
            row=dict(row)

            # -- Suppressing row[key] : 'nan'
            del_keys=[]
            for key , value in row.items():
                if str(value) in ['nan','NaT']:
                    del_keys.append(key)
            for key in del_keys :
                del row[key]


            l_new_data.append(cls(**row))


        return(l_new_data)

    # ______ Basic data queries

    # _______ Data ________


    def from_id(cls,_id): # could be define @classmethod in MongoObj class
        """
            Retrieve a live instance of MongoObj from its _id
        """
        def r_search(l_classes=[cls]):
            """
                Look in every register of subclasses recurssively
            """
            if l_classes :
                for cls_ in l_classes :
                    obj=cls_._register.get(ObjectId(_id))
                    if obj :
                        return(obj)
                    return(r_search(cls_.__subclasses__()))

        return(r_search())


    # ---- For iteration ----

    def invariant_lecture(cls,objs):
        """
            Method to overide for iterate according a lecture method (sort, not taking into account certain class)
        """
        return(objs)


    def __iter__(cls):
        objs=list(cls._register.values()).copy()
        return(iter(cls.invariant_lecture(objs)))

    # ______ For iteration _______

    def __len__(cls):
        """ Length of the class equal to the lenght of iterations THUS see sensible to defintion of invariant_lecture classmethod
        """
        return(len(cls._register))


    # ----- Storing the MongoObj localy ----

    def json_dumps(cls,folder_path='.dev'):

        file_name=f'{folder_path}/{cls.__name__}.json'

        kutility.core.create_file_path(file_name) # ensure that the folder structure to create the file is created


        with open(file_name,'w', encoding='utf-8') as f :
            json.dump([obj.__Mongo_dict__ for obj in cls]
                       ,f, ensure_ascii=False, indent=4)

    def json_loads(cls,folder_path='.dev',globals_=globals(),file_name=None):

        if not file_name :
            file_name=f'{folder_path}/{cls.__name__}.json'

        if not os.path.exists(file_name) :
            warnings.warn(f'Loading of {cls.__name__} failed ! no file {file_name}')
            return

        with open(file_name,'r') as f :
            data=json.load(f)
        for obj_doc in data :
            cls.create_instance(obj_doc,globals_)

    #_______ Storing the MongoObj localy _____





class MongoObj(object,metaclass=MetaID):

    def __new__(cls,_id=None,**kargs):

        if kargs.get('python_type'):
            if not kargs['python_type']==str(cls.__name__) :
                raise Exception(f"Try to load {kargs['python_type']} in obj of type {str(cls.__name__)}")

        if _id :
            _id=ObjectId(str(_id))

        instance=cls.from_id(_id)

        if not instance :
            instance=super().__new__(cls)
            instance._id=_id if _id else ObjectId()
            cls._register[instance._id]=instance

        return instance

    def init_att(self,att_name,att_value):
        """
            Rigth way to initialize attribute with not None value accordingly to a scheme especialy when they come from **rows
        """
        if att_value is not None : # Load new values (not None) of the data
            self.__dict__[att_name]=att_value
        if att_name not in self.__dict__ : # Look if the attribute is created and create it with none if necessary
            self.__dict__[att_name]=None

    @classmethod
    def invariant_lecture(cls,objs): # Overide of the MetaID
        """
            iterate according a lecture method :
                * only instance of this class (class or sub-classes)
                * sort according to generation_time of MongoObj
        """
        # ensure to iterate only on istance of the class (class and all sub-classes)
        objs=[obj for obj in objs if isinstance(obj,cls)]
        # sorte the iteration accoriding to generation_time
        objs=sorted(objs,key= lambda obj : obj.generation_time)
        return(objs)

    @classmethod
    def create_instance(cls,mongo_doc,globals_=None):
        """
            Dynamicaly create instance from a stored __Mongo_dict__ dictionary (stored in json file or mongodb)
        """
        # kclass=globals()["class_name"] ; instance=kclass()
        if not globals_ :
            globals_=globals()

        return(globals_[mongo_doc.get('python_type')](**mongo_doc))

    @property
    def __Mongo_dict__(self):
        """
            Storable version of the MongoObj (it replace in particular every reference to OTHER MongoObj by there _id)
        """

        def func(obj):
            # IF the object have an _id THEN replace the obj by a reference to its _id
            if isinstance(obj,ObjectId):
                return(str(obj))

            if isinstance(obj,MongoObj):
                warnings.warn('MongoObj should not contain other refrence them as ObjectId and use MongoObj()')
                return(str(obj._id))

            # IF Not an insatance of MongoObj implement __Mongo__dict___
            if isinstance(obj,object):
                if '__Mongo_dict__' in type(obj).__dir__(obj):
                    return(obj.__Mongo_dict__)

            return(obj)


        def rec_dumps(obj,func=func):
            if isinstance(obj,dict) :
                return({rec_dumps(key):rec_dumps(value) for key,value in obj.items()})
            elif isinstance(obj,list) :
                return([rec_dumps(e) for e in obj])
            else :
                return(func(obj))

        d=self.__dict__.copy()
        d['python_type']=str(type(self).__name__)

        return(rec_dumps(d))

    @property
    def generation_time(self):
        """
            Return the generation time of the obj from the one of its _id : ObjectId
        """
        return(ObjectId(self._id).generation_time)





# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"find_backward": "00_Core.ipynb",
         "find_data": "00_Core.ipynb",
         "create_file_path": "00_Core.ipynb",
         "download_data": "00_Core.ipynb",
         "save_authentification_session": "01_Selenium.ipynb",
         "set_auth_session_with_selenium": "01_Selenium.ipynb",
         "add_auth_session_web_driver": "01_Selenium.ipynb",
         "load_json": "01_Selenium.ipynb",
         "MetaID": "03_Structure.ipynb",
         "MongoObj": "03_Structure.ipynb",
         "update_settings_requirements": "05_cli.ipynb"}

modules = ["core.py",
           "selenium.py",
           "structure.py",
           "cli.py"]

doc_url = "https://Pyth314.github.io/kutility/"

git_url = "https://github.com/Pyth314/kutility/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: 00_Core.ipynb (unless otherwise specified).

__all__ = ['find_backward', 'find_data', 'create_file_path', 'download_data']

# Cell
import glob
import warnings
import urllib
import os

# Cell
def find_backward(folder='input',path='./'):
    """
        Go backward in the hierarchy to find the specified folder from path
    """
    res=[p for p in glob.glob(path+'*') if folder in p]
    if not res==[] :
        return res[0]
    return find_backward(folder,path=path+'../')

# Cell

def find_data(name,folder='input'):
    # Foward to find the specified data
    files=glob.glob(f"{find_backward(folder)}/{name}")
    files=glob.glob(f"{find_backward(folder)}/**/{name}")
    if len(files)==1:
        return(files[0])
    elif len(files)>1:
        warnings.warn(f'Warining : many files corespond : {files}\nFile path return : {files[0]}')
        return(files[0])
    else :
        print('No data found')

# Cell

def create_file_path(file_name):
    """Ensure that the right file structure is created before creating a file"""
    if not '/' in file_name :
        return None
    directories=file_name.split('/')[:-1]
    not_created=[]
    while not os.path.isdir('/'.join(directories+[''])):
        not_created.append(directories.pop())
        if not directories :
            break
    for _ in range(len(not_created)):
        directories.append(not_created.pop())
        os.mkdir('/'.join(directories+['']))


# Cell

def download_data(url,file_name,folder='download_data'):
    """Download everything urlretrieve and put it at the right place"""

    dir_path=f"{find_backward(folder='input')}/{folder}/"
    file_path=dir_path+file_name

    if not os.path.isdir(dir_path):
        os.mkdir(dir_path)

    if os.path.isfile(file_path):
        warnings.warn(f'Data {file_name} already in folder {folder}')
        return None


    create_file_path(file_path)

    urllib.request.urlretrieve(url,file_path)